/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : I2C_Master.c

 Hardware      : Applicable to ADuC7023 rev A later silicon
                 
 Description   : This I2C Master example shows how to initialise the I2C peripheral in Master mode.
 			This simple example writes 6 bytes to a Slave.
			It will read 6 bytes when on detecting an External IRQ on IRQ1
		
*********************************************************************/
#include <aduc7023.h>
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define I2C_RD_BIT 0x1

unsigned char szTxData[] = {0x1, 0x2, 0x10, 0x20, 0x40, 0x80};	// Array to send to Slave
unsigned char ucTxCount = 0;  				// Array index variable for szTxData[]
unsigned char szRxData[6];	  				// Array for reading Data from Slave
unsigned char ucRxCount = 0;  				// Array index variable for szRxData[]
unsigned char ucWaitingForXIRQ1 = 1; 		// Flag to begin reading from Slave - controlled by XIRQ1

unsigned int uiTest = 0;
int main(void) 
{
	POWKEY1 = 0x01;							// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	

	// Configure P0.4 and P0.5 for I2C mode
	GP0CON |= 0x00110000;					// Only write to bits 4/5
	GP0DAT |= 0x80000000;					//P0.7 as an output -- LED pin

	POWKEY3 = 0x76;			 			// Enable PWM, SPI, I2C0, I2C1 Peripherals with 41.78MHz clock
    POWCON1 = 0x924;					
	POWKEY4 = 0xB1;
	
	GP0DAT ^= 0x00800000;					// Complement P0.7
	// Enable I2C Master mode, baud rate and interrupt sources
	I2C0MCON = BIT0 + BIT4 					// Enable I2C Master + Enable Rx interrupt
			+ BIT5 + BIT8; 					// Enable Tx interrupt + Enable transmission complete interrupt
	I2C0DIV  = 0xC0C0;	   					// Select  clock rate
	
	uiTest = I2C0MCON;
	IRQEN = BIT8 + BIT16; 					// Enable I2C0 Slave and XIRQ1 interrupts

	// Begin Master Transmit sequence
	ucTxCount = 0;
	I2C0FSTA = BIT9;						// Flush Master Tx FIFO
	I2C0FSTA &= ~BIT9;
	I2C0MTX = szTxData[ucTxCount++];
	I2C0ADR0 = 0xA0; 
	ucWaitingForXIRQ1 = 1;
	ucRxCount = 0;
	while (1)
	{
	 	if (ucWaitingForXIRQ1 == 0)			// Wait for XIRQ0 to trigger - cleared in XIRQ0 ISR
		{
			ucWaitingForXIRQ1 = 1;
			ucRxCount = 0;
			I2C0MCNT0 = 5; 					// Read back 6 bytes
			I2C0ADR0 =  0xA1;				// + I2C_RD_BIT; // Begin Read sequence from Slave
		}
	}			
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int I2CMSTATUS = 0;

	IRQSTATUS = IRQSTA;	   					// Read off IRQSTA register

	if ((IRQSTATUS & BIT8) == BIT8)			//If I2C Master interrupt source
	{
		I2CMSTATUS = I2C0MSTA;
	   if ((I2CMSTATUS & BIT2) == BIT2) 	// If I2C Master Tx IRQ
	   {									
			if (ucTxCount < 6)		   		// Have max 6 bytes been sent?
				I2C0MTX = szTxData[ucTxCount++];	 // Load Tx buffer
	   }
	   if ((I2CMSTATUS & BIT3) == BIT3) 	// If I2C Master Rx IRQ
	   {
			if (ucRxCount < 6)		   		// Have max 6 bytes been received?
			{
				szRxData[ucRxCount] = I2C0MRX;  // Read Rx buffer
				ucRxCount++;
			}
	   }
	}
	if ((IRQSTATUS & BIT16) == BIT16)		//XIRQ1 interrupt source
	{
		ucWaitingForXIRQ1 = 0;				// Enable Read request from Slave.
		IRQCLR = BIT16;
	}
}

